/*
===================
JSHINT: set options
===================
*/

/*jshint esversion: 6*/
/*jshint sub:true*/

// Needs oolite >=1.82 (or, er, higher?)

/*
*/

this.name						=	"harderHermits";
this.author						=	"UK_Eliter";
this.copyright 					=	"See the license field";
this.description				=	"harderHermits.js";
this.license					=	"2014 CC-by-nc-sa 3.0";


/*
==================
DEBUGGING SWITCHES
==================
*/

// this.logging = true;
var consoleDebugMessages 		=	false; // do NOT comment this out. Rather, to disable, set to false.
this.debug 						=	false;
this.debug_testResurrection		=	false;


/*
========================
JSLINT: start of wrapper
========================
*/

( function () {

"use strict";


/*
=======
GLOBALS
=======
*/

this.name						=	"Harder Hermits";

this.abandoned_chanceAct 		=	0.15; 	// 0.15

this.hermit_chanceMoreEnergy	=	0.83; 	// 0.83.

this.installed_spicyHermits		=	false;	// gets changed to true if the OXP is installed.
this.installed_hermitage		=	false;	// gets changed to true if the OXP is installed.


/*
=========
DEBUGGING
=========
*/

this.$msg_dbg = function ( debugTxt )
{
	if ( !debugTxt ) { player.consoleMessage ( "Problem with debug message text!", 30 ); return; }
	player.consoleMessage ( this.name + " - " + debugTxt, 12);
	log ( this.name + " - DEBUG:", debugTxt);
};

this.$msg_info = function ( msgTxt )
{
	// if ( !msgTxt ) { player.consoleMessage ( "Problem with info message text!", 30 ); return; }
	log ( this.name + " - INFO:", msgTxt);
};


/*
================
SOUNDS
================
*/

this.mySound_hullBang = new SoundSource();
this.mySound_hullBang.sound = "hullbang.ogg";
// this sound built-in to Oolite - and does not need to be declared in the 'customsounds' file.


/*
================
ENTITY PICKER(S)
================

The following roles are from Oolite itself:

	rockhermit
	rockhermit-chaotic
	rock-hermit-pirate

The following role is from SpicyHermits:

	spicy_hermits_abandoned
*/


this.$isAbandonedHermit = function ( e )
{
	return e.isShip && ( e.primaryRole === "asteroid" && e.name === "Abandoned Rock Hermit" );
};

this.$isHermit = function ( e )
{
	return e.isShip && ( e.primaryRole === "rockhermit" || e.primaryRole === "rockhermit-chaotic" || e.primaryRole === "rockhermit-pirate" );
};

this.$isHermitButNotHermitage = function ( e )
{
	return e.isShip && ( e.primaryRole === "rockhermit" || e.primaryRole === "rockhermit-chaotic" || e.primaryRole === "rockhermit-pirate" ) && ! e.hasRole ( "hermitage" );
};


/*
======
TIMERS
======
*/

this.$timer_delete_hermitAction_warn = function ()
{
	if ( !this.timer_hermitAction_warn ) {return;}
	if ( this.timer_hermitAction_warn.isRunning ) {
		this.timer_hermitAction_warn.stop ();
	} delete this.timer_hermitAction_warn;
};

this.$timer_delete_hermitAction_warnThenExplode = function ()
{
	if ( !this.timer_hermitAction_warnThenExplode ) {return;}
	if ( this.timer_hermitAction_warnThenExplode.isRunning ) {
		this.timer_hermitAction_warnThenExplode.stop ();
	} delete this.timer_hermitAction_warnThenExplode;
};

this.$timer_delete_hermitAction_explode = function ()
{
	if ( !this.timer_hermitAction_explode ) {return;}
	if ( this.timer_hermitAction_explode.isRunning ) {
		this.timer_hermitAction_explode.stop ();
	} delete this.timer_hermitAction_explode;
};

this.$timer_delete_hermitAction_resurrect = function ()
{
	if ( !this.timer_hermitAction_resurrect ) {return;}
	if ( this.timer_hermitAction_resurrect.isRunning ) {
		this.timer_hermitAction_resurrect.stop ();
	} delete this.timer_hermitAction_resurrect;
};

this.$timer_delete_die = function ()
{
	if ( !this.timer_die ) {return;}
	if ( this.timer_die.isRunning ) {
		this.timer_die.stop ();
	} delete this.timer_die;
};

this.$timer_delete_all = function ()
{
	this.$timer_delete_die();
	this.$timer_delete_hermitAction_resurrect();
	this.$timer_delete_hermitAction_warn ();
	this.$timer_delete_hermitAction_warnThenExplode ();
	this.$timer_delete_hermitAction_explode ();
	if ( this.abandonedHermit ) {
		delete this.abandonedHermit;
	}
};


/*
===============
TIMED FUNCTIONS
===============
*/

this.$die_timed = function $die_timed()
{
	// The above format, with its repetition of the function name, is right. See http://aegidian.org/bb/viewtopic.php?p=257147#p257147.
	this.$timer_delete_die();
	// Do not automatically delete it, because it might not be running; other things can call this.
	if ( !player.ship ) { return; }
	// Next two lines are to avoid auto-eject shenanigans, etc.
	if ( player.ship.equipmentStatus( "EQ_AUTO_EJECT" ) === "EQUIPMENT_OK" ) {
		player.ship.removeEquipment( "EQ_AUTO_EJECT" );
	}
	if ( player.ship.equipmentStatus( "EQ_EEU" ) === "EQUIPMENT_OK" ) {
		player.ship.removeEquipment( "EQ_EEU" );
	}
	player.ship.explode(); 
};

this.$timed_hermitAction_message_warn = function ( e )
{
	if ( ! e || ! e.isValid ) { return ; }
	e.commsMessage ( "WARNING: This station will explode. Clear the area." );
	e.broadcastCascadeImminent ();
};

this.$timed_hermitAction_warn = function ()
{
	if ( this.debug ) { this.$msg_dbg ( "timed_hermitAction_warn." ); }
	this.$timer_delete_hermitAction_warn ();
	if ( ! this.abandonedHermit ) { return; }
	if ( ! this.abandonedHermit.isValid ) {
		delete this.abandonedHermit;
		return;
	}
	var e = this.abandonedHermit;
	this.$timed_hermitAction_message_warn ( e );
};

this.$timed_hermitAction_warnThenExplode = function ()
{
	if ( this.debug ) { this.$msg_dbg ( "timed_hermitAction_warnThenExplode." ); }
	this.$timer_delete_hermitAction_warnThenExplode ();
	if ( ! this.abandonedHermit ) { return; }
	if ( ! this.abandonedHermit.isValid ) {
		delete this.abandonedHermit;
		return;
	}
	this.$timed_hermitAction_message_warn ( this.abandonedHermit );
	var seconds = ~~( ( Math.random () * 60 ) + 0.5 ); // * 60
	this.timer_hermitAction_explode = new Timer ( this, this.$timed_hermitAction_explode, seconds );
};

this.$timed_hermitAction_explode = function ()
{
	if ( this.debug ) { this.$msg_dbg ( "timed_hermitAction_explode." ); }
	this.$timer_delete_hermitAction_explode ();
	if ( ! this.abandonedHermit ) { return; }
	if ( ! this.abandonedHermit.isValid ) {
		delete this.abandonedHermit;
		return;
	}
	this.abandonedHermit.explode ();
	// NB SpicyHermits occasionally adds a q-mine death action!
	delete this.abandonedHermit;
};

this.$timed_hermitAction_resurrect = function ()
{
	if ( this.debug ) { this.$msg_dbg ( "timed_hermitAction_resurrect." ); }
	this.$timer_delete_hermitAction_resurrect ();
	if ( ! this.abandonedHermit ) { return; }
	if ( ! this.abandonedHermit.isValid ) {
		delete this.abandonedHermit;
		return;
	}

	// We have to *replace* the hermit (otherwise it cannot be a station).
	// If the hermit is close enough to the player to notice the change, abort.
	if ( ! player.ship || ! player.ship.position ) {
		delete this.abandonedHermit;
		return;
	}
	var ps = player.ship; // for speed, but changes won't affect the original variable
	if ( this.abandonedHermit.position.distanceTo(ps.position) < 50000 ) {
		if ( this.debug ) { this.$msg_dbg ( "Hermit too close to player for resurrection." ); }
		delete this.abandonedHermit;
		return;
	}

	// Get data for the replacement.
	var orientation = this.abandonedHermit.orientation;
	var position = this.abandonedHermit.position;

	if ( this.abandonedHermit ) {
		this.abandonedHermit.remove( true );
		delete this.abandonedHermit;
	}

	// Do the replacement.
	// Note that Spicy Hermits overrides Oolite's rockhermit shipdata, so what we'll get here is 'spicy' stuff.

	var replacement;
	replacement = system.addShips( "harderHermits_resurrected", 1, position );

	// this.replacement = this.abandonedHermit.spawnOne( "harderHermits_resurrected" );
	
	if ( replacement[0] && replacement[0].isValid ) {
		if ( this.debug ) { this.$msg_dbg ( "Created replacement." ); }
		// this.replacement.orientation = this.abandonedHermit.orientation; 
		// this.replacement.position = this.abandonedHermit.position;
		replacement[0].orientation = orientation;
		replacement[0].lightsActive = false;
		replacement[0].desiredRange = 13000; // I am unsure this will do anything - I should TEST it.
		if ( this.debug ) { replacement[0].displayName = "Abandoned rock hermit (RESURRECTED)"; }
		// Energy
		var r = Math.random ();
		replacement[0].maxEnergy *= ( 1.5 + r );
	} else if ( this.debug ) { this.$msg_dbg ( "Replacement failed to exist." ); }
	};


/*
==============
EVENT HANDLERS
==============
*/

this.startUp = function ()
{
	if ( this.debug ) {
		// DEBUGGING is ON.
		this.$msg_dbg ( "DEBUGGING is ON." );
		if ( this.debug_testResurrection ) {
			this.$msg_dbg ( "Testing resurrection." );
		}
		if (this.$roleExists( "hermitage" )) {
			// I have trouble detecting the hermitage OXP in any other way, for some reason.
			this.installed_hermitage = true;
			this.$msg_info ( "Detected hermitage OXP.");
		} else {
			this.$msg_dbg ( "Did NOT detect hermitage OXP.");
		}
		if ( worldScripts [ "spicy_hermits_abandoned" ] ) {
			this.installed_spicyHermits = true;
			this.$msg_info ( "Detected SpicyHermits OXP." );
		} else {
			this.$msg_dbg ( "Did NOT detect SpicyHermits OXP." );
		}
	} else {
		// DEBUGGING is OFF.
		if (this.$roleExists( "hermitage" )) {
		// I have trouble detecting the hermitage OXP in any other way, for some reason.
			this.installed_hermitage = true;
			this.$msg_info ( "Detected hermitage OXP." );
		}
		if ( worldScripts [ "spicy_hermits_abandoned" ] ) {
			this.installed_spicyHermits = true;
			this.$msg_info ( "Detected SpicyHermits OXP." );
		}
	}

};

this.shipDockedWithStation = function (station)
{
	var d, r;

	if ( player.ship.docked && player.ship.dockedStation.name === "Abandoned Rock Hermit" ) {
		// Unmodified abandoned hermits from SpicyHermits do not allow the player to dock (at all), so this routine won't fire for that.
		this.showScreen = "rejectPage";
		mission.runScreen
		(
			{
				title: "Abandoned Rock Hermit (well, not really abandoned)",
				color: "redColor",
				message: "You have docked at a disguised pirate cove.\n\nNine out of ten for style, but minus several thousand for good thinking.",
				model: "rockhermit",
				choicesKey: "harderHermits_docked"
			},
			function (choice) {
				if (choice === "1_RETURN") {
					r = Math.random();
					// r = 0.1; // TESTING
					if ( r < 0.2 ) {
						player.commsMessage( "The pirates attack your launching ship!" );
						player.ship.launch();
						if (!this.timer_die) {
							this.timer_die = new Timer( this,this.$die_timed, 1.7 );
						}
						this.mySound_hullBang.play();
					} else if ( r < 0.8 ) {
						d = 40 + ~~( ( Math.random() * 500 ) + 1 );
						player.commsMessage( "The pirates attack your launching ship!" );
						player.ship.launch();
						player.ship.energy = player.ship.energy - d;
						if ( player.ship.energy < 0 ) {
							if (!this.timer_die) {
								this.timer_die = new Timer( this,this.$die_timed, 1.7 );
							}
						} else {
							this.mySound_hullBang.play();
						}
					}
					station.reactToAIMessage( "ATTACKED" );
				}
			}
		);
	}
};



this.playerWillEnterWitchspace = function()
{
	this.$timer_delete_all ();
};

this.shipExitedWitchspace = function ()
{
	if ( system.isInterstellarSpace ) { return; }
	// log("Harder Hermits", "DEBUG: #1.");

	// TREAT NORMAL HERMITS
	// (as Vs the abandoned ones supplied by SpicyHermits and as Vs the 'quasi-abandoned' ones that this OXP can derive from SpicyHermits).

	var s;
	if ( this.installed_hermitage ) {
		s = system.filteredEntities ( this, this.$isHermitButNotHermitage );
	} else {
		s = system.filteredEntities ( this, this.$isHermit );
	}

	var l = s.length;
	if ( l === 0 ) { return; }
	var i;
	var oldMaxEnergy;
	var r;
	var str;
	// SpicyHermits gives pirate rock-hermits 2500 maxEnergy, and leaves the maxEnergy of other types alone, at 1,000.
	// SpicyHermits saves the position of its hermits but anything else - including their maxEnergy - about them.
	// We will not try to save that, either.
	if ( this.installed_spicyHermits ) {
		i = l;
		while ( i --) {
			if ( s[i] && s[i].isValid ) {
				r = Math.random ();
				if ( r < this.hermit_chanceMoreEnergy ) {
					r = ~~( ( r * 10 ) + 0.5 ) / 10;
					if ( s[i].primaryRole !== "rockhermit-pirate" ) {
						r += 0.6;
					} else if ( r > 0.3 ) {
						r -= 0.1;
					}
					// Tend to increase energy of the *non-pirates* more.
					oldMaxEnergy = s[i].maxEnergy;
					s[i].maxEnergy *= ( 1 + r );
					// this.$msg_info ( "Harder Hermits", "FOR INFO: Increased the energy of a rock hermit." );
					str = "FOR INFO: Increased the energy of spicy rock hermit <" + s[i].displayName + "> from " + oldMaxEnergy + " to " + s[i].maxEnergy;
					if ( this.debug ) { this.$msg_dbg ( str ); }
				}
			}
		}
	} else {
		// SpicyHermits not installed.
		while (i--) {
			if ( s[i] && s[i].isValid ) {
				r = Math.random ();
				if ( r < this.hermit_chanceMoreEnergy ) {
					r = ~~( ( r * 10 ) + 0.5 ) / 10;
					oldMaxEnergy = s[i].maxEnergy;
					s[i].maxEnergy *= ( 1 + r );
					// this.$msg_info ( "Harder Hermits", "FOR INFO: Increased the energy of a rock hermit." );
					str = "FOR INFO: Increased the energy of rock hermit <" + s[i].displayName + "> from " + oldMaxEnergy + " to " + s[i].maxEnergy;
					if ( this.debug ) { this.$msg_dbg ( str ); }
				}
			}
		}
	}

	// CONSIDER TREATING ANY ABANDONED HERMITS
	if ( ! this.installed_spicyHermits ) { return; }

	if ( ! this.debug_testResurrection ) {
		r = Math.random ();
		if ( r > this.abandoned_chanceAct ) {
			if ( this.debug ) {
				this.$msg_dbg ( "Chance prevented proceeding with any abandoned hermits." );
				return;
			}
		}
	}

	if ( this.debug ) { this.$msg_dbg ( "Seeking abandoned hermits . ." ); }
	s = system.filteredEntities ( this, this.$isAbandonedHermit );
	l = s.length;
	if ( l < 1 ) {
		if ( this.debug ) { this.$msg_dbg ( ".. found no abandoned hermits." ); }
		return;
	}
	if ( this.debug ) { this.$msg_dbg ( ". . found at least one abandoned hermit." ); }
	// Pick one of them.
	i = ~~ ( Math.random () * l );
	// Check there is nothing fundamentally wrong with it.
	if ( ! (s [i] && s[i].isValid ) ) {
		if ( this.debug ) { this.$msg_dbg ( "Invalid abandoned hermit selected." ); }
		return;
	}
	// Set some of its properties.
	if ( this.debug ) { s[i].displayName = "Abandoned Rock Hermit (TWEAKED)"; }
	this.abandonedHermit = s[i]; // Set a global variable because the timed routines will need it.

	var seconds;
	if ( this.debug_testResurrection ) {
		seconds = 5;
		this.timer_hermitAction_resurrect = new Timer ( this, this.$timed_hermitAction_resurrect, seconds );
		this.$msg_dbg ( "Set abandoned hermit to mode <resurrect>." );
	} else {
		seconds = ~~( ( Math.random () * 10800 ) + 1 ) + 5; // * 10800 + 5
		r = Math.random ();
		if ( r < 0.7 ) {
			// ** Mode: warn then explode **
			this.timer_hermitAction_warnThenExplode = new Timer ( this, this.$timed_hermitAction_warnThenExplode, seconds );
			if ( this.debug ) { this.$msg_dbg ( "Set abandoned hermit to mode <warn-then-explode>." ); }
			
		} else if ( r < 0.75 ) {
			// ** Mode: warn **
			this.timer_warn = new Timer ( this, this.$timed_hermitAction_warn, seconds );
			if ( this.debug ) { this.$msg_dbg ( "Setting abandoned hermit to mode <warn>." ); }
			
		} else if ( r < 0.8 ) {
			// ** Mode: explode **
			this.timer_hermitAction_explode = new Timer ( this, this.$timed_hermitAction_explode, seconds );
			if ( this.debug ) { this.$msg_dbg ( "Set abandoned hermit to mode <explode>." ); }
		} else {
			// ** Mode: resurrect **
			this.timer_hermitAction_resurrect = new Timer ( this, this.$timed_hermitAction_resurrect, seconds );
			if ( this.debug ) { this.$msg_dbg ( "Set abandoned hermit to mode <resurrect>." ); }
		}
	}

	if ( this.debug ) {
		this.$msg_info ( "Primed an abandoned rock hermit." );
		str = "Primed hermit <" + this.abandonedHermit.displayName + "> for action in " + seconds + " seconds time.";
		this.$msg_dbg ( str );
	}
};

/*
=======================
MISC
=======================
*/

this.$roleExists = function (roleString) 
{
	var dummyShips = system.addShips(roleString,1,[100000,100000,100000]);
	if (dummyShips == null) {return false;}
	var exists = (dummyShips.length == 1);
	if (exists) {if (dummyShips[0].isValid) {dummyShips[0].remove(true);}}
	return exists;
};


/*
=======================
JSHINT: end of wrapper
=======================
*/

}).call ( this );


// EOF
